package types

import (
	"net"
	"sync"
)

type discoverTestZoneCallback func(host *DiscoveryZone)

type DiscoveryZone struct {
	ID             int                `json:"id,omitempty"`
	IP             string             `json:"ip"`
	Netmask        string             `json:"mask"`
	Iface          string             `json:"iface"`
	Mac            string             `json:"mac"`
	FirstScanRange int64              `json:"firstScanRange"`
	LastScanRange  int64              `json:"lastScanRange"`
	Histories      []*HostScanHistory `json:"-"`
	Hosts          []*DiscoveryHost   `json:"hosts"`
	CurrentTypes   string             `json:"-"`

	Hosts_ map[string]*DiscoveryHost `json:"-"`
	M      *sync.RWMutex             `json:"-"`
}

func (z *DiscoveryZone) SetHost(ip string, value *DiscoveryHost) {
	z.M.Lock()
	defer z.M.Unlock()
	z.Hosts_[ip] = value
}

func (z *DiscoveryZone) GetHost(ip string) (*DiscoveryHost, bool) {
	z.M.RLock()
	defer z.M.RUnlock()
	h, ok := z.Hosts_[ip]
	return h, ok
}

func (p *DiscoveryZone) AddHistory(h *HostScanHistory) {
	p.Histories = append(p.Histories, h)
}

func (z *DiscoveryZone) CidrString() string {
	netmask := net.IPMask(net.ParseIP(z.Netmask))
	str := z.IP + "/" + netmask.String()

	_, ipnet, err := net.ParseCIDR(str)
	if nil != err {
		//logging.Logger.Error(fmt.Sprintf("ParseCIDR: %v", err))
	}

	return ipnet.String()
}
